# -*- coding: utf-8 -*-
# Copyright 2022-2023 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.
import json
import os.path
import subprocess

import pytest

import ucsschool_id_connector.plugin_loader

from conftest import (  # isort:skip  # noqa: E402
    set_school_authority_on_id_connector,
)

# load ID Broker plugin
ucsschool_id_connector.plugin_loader.load_plugins()
id_broker = pytest.importorskip("idbroker")
pytestmark = pytest.mark.id_broker

from idbroker.initial_sync import CONFIG_PATH as SCHOOL_CONFIG_PATH, get_ous  # isort:skip  # noqa: E402

SCRIPT_PATH = (
    "/var/lib/univention-appcenter/apps/ucsschool-id-connector/conf/"
    "plugins/packages/idbroker/manage_schools_to_sync.py"
)


@pytest.mark.asyncio
@pytest.mark.parametrize("all_schools", (True, False), ids=lambda x: f"all_schools-{x}")
async def test_add_schools(
    id_broker_school_auth_conf,
    make_school_authority,
    make_sender_user_special,
    kelvin_school_on_sender,
    make_kelvin_workgroup_on_id_connector,
    make_kelvin_school_class_on_id_connector,
    wait_for_kelvin_user_on_id_broker,
    wait_for_kelvin_school_class_on_id_broker,
    wait_for_kelvin_workgroup_on_id_broker,
    all_schools,
    schedule_schools_delete_on_id_broker,
    delete_school_on_id_broker,
):
    # create school with users & groups
    # remove users & groups from id broker
    # add school
    school_authority = await set_school_authority_on_id_connector(
        id_broker_school_auth_conf, make_school_authority, schools=[kelvin_school_on_sender.name]
    )
    await schedule_schools_delete_on_id_broker(
        school_authority=school_authority, schools=[kelvin_school_on_sender.name]
    )
    sender_user = await make_sender_user_special(
        ous=[kelvin_school_on_sender.name],
    )
    await wait_for_kelvin_user_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=sender_user["name"],
    )
    school_class = await make_kelvin_school_class_on_id_connector(
        school_name=kelvin_school_on_sender.name, users=[sender_user["name"]]
    )
    await wait_for_kelvin_school_class_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=school_class.name,
    )
    work_group = await make_kelvin_workgroup_on_id_connector(
        school_name=kelvin_school_on_sender.name, users=[sender_user["name"]]
    )
    await wait_for_kelvin_workgroup_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=work_group.name,
    )
    await delete_school_on_id_broker(config=school_authority, schools=[kelvin_school_on_sender.name])
    if all_schools:
        add_school_args = [
            SCRIPT_PATH,
            "add_schools",
            "--num_tasks",
            "5",
            "--school_authority",
            school_authority.name,
            "--all_schools",
        ]
    else:
        add_school_args = [
            SCRIPT_PATH,
            "add_schools",
            "--num_tasks",
            "5",
            "--school_authority",
            school_authority.name,
            kelvin_school_on_sender.name,
        ]
    # should fail, because there we already added the same school in set_school_authority_on_id_connector
    with pytest.raises(subprocess.CalledProcessError):
        subprocess.check_call(add_school_args)
    subprocess.check_call(
        add_school_args
        + [
            "--force",
        ]
    )
    await wait_for_kelvin_user_on_id_broker(
        s_a_name=school_authority.name, school=kelvin_school_on_sender.name, name=sender_user["name"]
    )
    sc = await wait_for_kelvin_school_class_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=school_class.name,
    )
    assert sc.users == [f"{school_authority.name}-{sender_user['name']}"]
    wg = await wait_for_kelvin_workgroup_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=work_group.name,
    )
    assert wg.users == [f"{school_authority.name}-{sender_user['name']}"]
    with open(os.path.join(SCHOOL_CONFIG_PATH, f"{school_authority.name}.json")) as fin:
        config = json.load(fin)
        assert kelvin_school_on_sender.name in config["plugin_configs"]["id_broker"]["schools"]


@pytest.mark.asyncio
@pytest.mark.parametrize("all_schools", (True, False), ids=lambda x: f"all_schools-{x}")
async def test_remove_schools(
    id_broker_school_auth_conf,
    make_school_authority,
    make_sender_user_special,
    kelvin_school_on_sender,
    make_kelvin_workgroup_on_id_connector,
    make_kelvin_school_class_on_id_connector,
    wait_for_kelvin_user_on_id_broker,
    wait_for_kelvin_school_class_on_id_broker,
    wait_for_kelvin_workgroup_on_id_broker,
    all_schools,
    schedule_schools_delete_on_id_broker,
):
    school_authority = await set_school_authority_on_id_connector(
        id_broker_school_auth_conf, make_school_authority, schools=[kelvin_school_on_sender.name]
    )
    await schedule_schools_delete_on_id_broker(
        school_authority=school_authority, schools=[kelvin_school_on_sender.name]
    )
    work_group = await make_kelvin_workgroup_on_id_connector(school_name=kelvin_school_on_sender.name)
    await wait_for_kelvin_workgroup_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=work_group.name,
    )
    user = await make_sender_user_special(
        ous=[kelvin_school_on_sender.name],
    )
    await wait_for_kelvin_user_on_id_broker(
        s_a_name=school_authority.name, school=kelvin_school_on_sender.name, name=user["name"]
    )
    school_class = await make_kelvin_school_class_on_id_connector(
        school_name=kelvin_school_on_sender.name, users=[user["name"]]
    )
    await wait_for_kelvin_school_class_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=school_class.name,
    )
    remove_school_args = [
        SCRIPT_PATH,
        "remove_schools",
        "--school_authority",
        school_authority.name,
    ]
    if all_schools:
        remove_school_args += ["--all_schools"]
        subprocess.check_call(remove_school_args)
    else:
        remove_school_args += [kelvin_school_on_sender.name]
        subprocess.check_call(remove_school_args)
    await wait_for_kelvin_user_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=user["name"],
        must_exist=False,
    )
    await wait_for_kelvin_school_class_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=school_class.name,
        must_exist=False,
    )
    await wait_for_kelvin_workgroup_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=work_group.name,
        must_exist=False,
    )
    with open(os.path.join(SCHOOL_CONFIG_PATH, f"{school_authority.name}.json")) as fin:
        config = json.load(fin)
        assert kelvin_school_on_sender.name not in config["plugin_configs"]["id_broker"]["schools"]
        if all_schools:
            assert config["plugin_configs"]["id_broker"]["schools"] == []
        else:
            assert kelvin_school_on_sender.name not in config["plugin_configs"]["id_broker"]["schools"]
    with pytest.raises(subprocess.CalledProcessError) as exc:
        subprocess.check_call(remove_school_args)
        assert exc.code == 1


@pytest.mark.asyncio
async def test_remove_schools_dont_remove_schools_on_id_broker(
    id_broker_school_auth_conf,
    make_school_authority,
    make_sender_user_special,
    kelvin_school_on_sender,
    make_kelvin_workgroup_on_id_connector,
    make_kelvin_school_class_on_id_connector,
    wait_for_kelvin_user_on_id_broker,
    wait_for_kelvin_school_class_on_id_broker,
    wait_for_kelvin_workgroup_on_id_broker,
    schedule_schools_delete_on_id_broker,
):
    school_authority = await set_school_authority_on_id_connector(
        id_broker_school_auth_conf, make_school_authority, schools=[kelvin_school_on_sender.name]
    )
    await schedule_schools_delete_on_id_broker(
        school_authority=school_authority, schools=[kelvin_school_on_sender.name]
    )
    work_group = await make_kelvin_workgroup_on_id_connector(school_name=kelvin_school_on_sender.name)
    user = await make_sender_user_special(
        ous=[kelvin_school_on_sender.name],
    )
    school_class = await make_kelvin_school_class_on_id_connector(
        school_name=kelvin_school_on_sender.name, users=[user["name"]]
    )
    await wait_for_kelvin_user_on_id_broker(
        s_a_name=school_authority.name, school=kelvin_school_on_sender.name, name=user["name"]
    )
    await wait_for_kelvin_school_class_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=school_class.name,
    )
    await wait_for_kelvin_workgroup_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=work_group.name,
    )
    remove_school_args = [
        SCRIPT_PATH,
        "remove_schools",
        "--school_authority",
        school_authority.name,
        "--delete_schools",
        "false",
        kelvin_school_on_sender.name,
    ]
    subprocess.check_call(remove_school_args)
    await wait_for_kelvin_user_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=user["name"],
    )
    await wait_for_kelvin_school_class_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=school_class.name,
    )
    await wait_for_kelvin_workgroup_on_id_broker(
        s_a_name=school_authority.name,
        school=kelvin_school_on_sender.name,
        name=work_group.name,
    )
    with open(os.path.join(SCHOOL_CONFIG_PATH, f"{school_authority.name}.json")) as fin:
        config = json.load(fin)
        assert kelvin_school_on_sender.name not in config["plugin_configs"]["id_broker"]["schools"]


@pytest.mark.asyncio
@pytest.mark.parametrize("all_schools", (True, False), ids=lambda x: f"all_schools-{x}")
async def test_configure_schools(
    id_broker_school_auth_conf,
    make_school_authority,
    kelvin_school_on_sender,
    all_schools,
    schedule_schools_delete_on_id_broker,
):
    school_authority = await set_school_authority_on_id_connector(
        id_broker_school_auth_conf, make_school_authority, schools=[]
    )
    schools_on_sender = None
    if all_schools:
        schools_on_sender = [school.name for school in await get_ous("*")]
        configure_all_school_args = [
            SCRIPT_PATH,
            "add_schools",
            "--school_authority",
            school_authority.name,
            "--initial_sync",
            "false",
            "--all_schools",
        ]
    else:
        configure_all_school_args = [
            SCRIPT_PATH,
            "add_schools",
            "--school_authority",
            school_authority.name,
            "--initial_sync",
            "false",
            kelvin_school_on_sender.name,
        ]
        await schedule_schools_delete_on_id_broker(
            school_authority=school_authority, schools=[kelvin_school_on_sender.name]
        )
    subprocess.check_call(configure_all_school_args)
    with open(os.path.join(SCHOOL_CONFIG_PATH, f"{school_authority.name}.json")) as fin:
        config = json.load(fin)
        if all_schools:
            assert set(schools_on_sender) == set(config["plugin_configs"]["id_broker"]["schools"])
        else:
            assert [kelvin_school_on_sender.name] == config["plugin_configs"]["id_broker"]["schools"]
    # -> should fail, because there we try to add the same schools again.
    with pytest.raises(subprocess.CalledProcessError):
        subprocess.check_call(configure_all_school_args)
    # -> should pass with option force
    subprocess.check_call(
        configure_all_school_args
        + [
            "--force",
        ]
    )
